require "rd/rd2html-lib"
require 'rd/rdfmt'
require 'amrita2/template'

module Amrita2
  include Amrita2
  RDTmpl = TemplateText.new <<-END
  <html>
  <head id='header'>
     <title id='title' />
     <link id='css' rel="stylesheet" type="text/css" />
     <meta id='lang' content="text/html; charset=euc-jp" http-equiv="content-type" />
  </head>
  <body id='document'>
    <span id='output'></span>
    <div id='block'>
      <h1 id='header1'> header</h1>
      <h2 id='header2'> header</h2>
      <h3 id='header3'> header</h3>
      <p id='textblock'> </p>
      <ul id='list'><li id='item'></li></ul>
      <ol id='enumlist'><li id='item'></li></ol>
      <dl id='desclist'><dt id='term' /><dd id='description' /></dl>
    </div>
    <div id='inline'>
      <em id='emphasis'>*<span id='text'></span>*</em>
      <code id='code'><span id='text'></span></code>
      <var id='var'><span id='text'></span></var>
      <kbd id='kbd'><span id='text'></span></kbd>
      <a id='link_to_url'></a>
    </div>
  </body>
  </html>
  END
  RDTmpl.define_parts(:block=>[:header1, :header2, :header3, :textblock, 
                        { :list=>:item }, 
                        { :enumlist=>:item } , 
                        { :desclist=>[:term, :description] } ],
                      :inline=>[:emphasis, :code, :var, :kbd, :link_to_url])

  class Amrita2Visitor < RD::RD2HTMLVisitor
    include Amrita2
    include Amrita2::Runtime
    include Amrita2::Core

    attr_accessor :option
    def initialize(doctmpl, blocktmpl, inlinetmpl, extention_processor, option=nil)
      @doctmpl, @blocktmpl, @inlinetmpl, @extention_processor =
        doctmpl, blocktmpl, inlinetmpl, extention_processor
      @option = option
    end

    def visit_DocumentElement(element)
      @doctmpl.expand(out="") do |m|
        if @option
          m.header do |mh|
            mh.title(@option[:title]) if @option[:title]
            mh.css(nil,:href=>@option[:css]) if @option[:css]
            mh.lang(nil,:content=>"text/html; charset=#{@option[:lang]}") if @option[:lang]
          end
        end
        m.document do |md|
          md.output(visit_children(element))
        end
      end
      out
    end

    def visit_children(element)
      SanitizedString[super.to_s]
    end

    def visit_TextBlock(element)
      if (is_this_textblock_only_one_block_of_parent_listitem?(element) or
	  is_this_textblock_only_one_block_other_than_sublists_in_parent_listitem?(element))
        visit_children(element).gsub(/\s/, " ")
      else
        @blocktmpl[:textblock][visit_children(element)]
      end
    end
    HeadlineMethods = [
      nil,
      :header1,
      :header2,
      :header3,
      :header4,
      :header5,
      :header6,
      nil
    ]

    def visit_Headline(element)
      c = visit_children(element)
      @blocktmpl[HeadlineMethods[element.level]][c]
    end

    def visit_ItemList(element)
      @blocktmpl[:list][visit_children(element)]
    end

    def visit_ItemListItem(element)
      @blocktmpl[:list][:item][visit_children(element)]
    end

    def visit_EnumList(element)
      @blocktmpl[:enumlist][visit_children(element)]
    end

    def apply_to_EnumListItem(element, content)
      @blocktmpl[:enumlist][:item][content]
    end

    def visit_DescList(element)
      @blocktmpl[:desclist][visit_children(element)]
    end

    def apply_to_DescListItem(element, term, description)
      @blocktmpl[:desclist][:term][term]  +
      @blocktmpl[:desclist][:description][description] 
    end

    def visit_StringElement(element)
      element.to_label.amrita_sanitize
    end

    def visit_Emphasis(element)
      @inlinetmpl[:emphasis][:text=>visit_children(element)]
    end

    def visit_Code(element)
      @inlinetmpl[:code][:text=>visit_children(element)]
    end

    def visit_Var(element)
      @inlinetmpl[:var][:text=>visit_children(element)]
    end

    def visit_Keyboard(element)
      #@inlinetmpl[:kbd][:text=>visit_children(element)]
      name, *a = visit_children(element).split(' ')
      process_ext(name, a)
    end

    def visit_Reference(element)
      case element.label
      when RD::Reference::RDLabel
        name, *a = element.to_label.split(':')
        process_ext(name,  *a)
      when  RD::Reference::URL
        @inlinetmpl[:link_to_url][
          a(:href=>element.label.url) do 
            visit_children(element)
          end
        ]
      else
        raise "can't happen"
      end
    end
    
    def apply_to_MethodListItem(element, term, content)
      name, *args =  split_MethodListparams(term)
      args.unshift content 
      @extention_processor.send(name.intern, args) 
    end

    def split_MethodListparams(term)
      ret = []
      while term.size > 0
        case term 
        when  /^\s*\"([^"]+)\"(.*)/ #'"
          ret << $1
          term = $2
        when /^\s*(\S+)(.*)/
          ret << $1
          term = $2
        else
          raise 'can not happen'
        end
      end
      ret
    end

    def apply_to_MethodList(element, items)
      items
    end

    def process_ext(name, *args)
      @extention_processor.send(name.intern, *args)
    end
  end

  class RDAStandardExtention
    include Amrita2
    include Amrita2::Core
    include Amrita2::Runtime
    extend Amrita2::Runtime

    EXTTMPL = TemplateText.new <<-END
    <div>
      <a id='rubytalk' href="http://blade.nagaokaut.ac.jp/cgi-bin/scat.rb/ruby/ruby-talk/$1">[ruby-talk:$1]</a>
      <a id='raa' href="http://raa.ruby-lang.org/list.rhtml?name=$1">[$2]</a>]
      <div id='include'><pre>$1$0</pre></div>
      <amrita:erb id='blockquote'>
         <blockquote cite='<%= $_[1] %>'>
            <p><%= $_[0] %></p><p>(<a href='<%= $_[1] %>'><%= $_[2] %></a>)</p>
         </blockquote>
      </amrita:erb>
    </div>
    END

    EXTTMPL.use_erb(binding)
    EXTTMPL.element_option[:rubytalk] = :use_args
    EXTTMPL.element_option[:raa] = :use_args
    EXTTMPL.element_option[:include] = :use_args
    EXTTMPL.define_parts(:rubytalk, :raa, :blockquote, :include)

    def initialize(exttmpl=nil)
      @exttmpl = (exttmpl || EXTTMPL)
    end

    def table(name, cols, *args)
      content = yield
      cols = cols.to_i
      items = content.split
      ret = self.send(name) do |mt|
        mt.header do |mh|
          cols.times do |i|
            mh.header_item(items.shift) 
          end
        end
        lines = items.size/cols
        lines.times do |i|
          mt.detail(i) do |md|
            md.header_item(items.shift)
            (cols-1).times do |j|
              md.item(items.shift)
            end
          end
        end
      end
      ret
    end

    def include(name, title=name)
      super("--- #{title} ---\n") { File::open(name).read }
    end

    def method_missing(sym , *arg, &block)
      arg = t(*arg)
      tmpl = @exttmpl[sym]
      raise "extention template element #{sym} not found" unless tmpl
      tmpl.expand(out="") do |m|
        meth = m.method(sym)
        begin
          meth.call(*arg, &block)
        end
      end
    end

    def method_missing_x(sym , *arg, &block)
      arg = t(*arg)
      tmpl = @exttmpl[sym]
      raise "extention template element #{sym} not found" unless tmpl
      if block_given?
        if arg.size == 0
          tmpl.expand(out="") do |m|
            m.send(sym, arg.first, &block)
          end
          out
        else
          tmpl.expand(out="") do |m|
            m.send(sym, *arg, &block)
          end
          out
        end
      else
        if arg.size == 0
          tmpl[arg.first]
        else
          tmpl[arg]
        end
      end
    end
  end

  class RDAProcessor
    include Amrita2
    def self.compile_rd(path, out=nil, opts={})
      out = path + ".html" unless out
      css = opts[:css] 
      File::open(path) do |i|
        File::open(out, "w") do |out|
          src = i.read
          ret = Amrita2::RDAProcessor.new(opts).process(src, opts)
          out.print ret
        end
      end
    end

    def initialize(opts={})
      doc = opts[:doc_template] || RDTmpl
      block = opts[:block_template] || RDTmpl[:block]
      inline = opts[:inline_template] || RDTmpl[:inline]
      ext = opts[:extention_processor] || RDAStandardExtention.new(opts[:exttmpl])
      @visitor = Amrita2Visitor.new(doc, block, inline, ext)
    end

    def process(rdtext, opt=nil)
      tree = RD::RDTree.new("=begin\n#{rdtext}\n=end\n\n")
      @visitor.option = opt
      @visitor.visit(tree)
    end
  end
end
