require 'test/unit'
require 'test/unit/testsuite'
require 'test/unit/ui/console/testrunner'
require 'amrita2/rd'
require 'rexml/document'
require 'util'

module Amrita2RDTest
  class TestRD < Test::Unit::TestCase
    include Amrita2

    def test_sanitize
      s = "\na<b>c\n"
      ret = RDAProcessor.new.process(s)
      assert_equal_x("<html><body><p>a&lt;b&gt;c\n</p></body></html>",ret)
    end

    def test_simple
      s = 'a'
      ret = RDAProcessor.new.process(s)
      assert_equal_x("<html><body><p>a\n</p></body></html>",ret)
    end

    def test_em
      s = '((*a*))'
      ret = RDAProcessor.new.process(s)
      assert_equal_x("<html><body><p><em>*a*</em></p></body></html>",ret)
    end

    def test_inline
      s = '(({code}))'
      ret = RDAProcessor.new.process(s)
      #assert_equal_x("<html><body><p><code>+code+</code><var>_var_</var><kbd>|key|</kbd></p></body></html>",ret)
      assert_equal_x("<html><body><p><code>code</code></p></body></html>",ret)

      s = '(({code}))((|var|))'
      ret = RDAProcessor.new.process(s)
      assert_equal_x("<html><body><p><code>code</code><var>var</var></p></body></html>",ret)
    end

    def test_url
      s = '((<ruby home|URL:http://www.ruby-lang.org>))'
      ret = RDAProcessor.new.process(s)
      assert_equal_x("<html><body><p><a href='http://www.ruby-lang.org'>ruby home</a></p></body></html>",ret)
    end

    def test_url2
      s = '((<rubytalk:12345>))'
      ret = RDAProcessor.new.process(s)
      assert_equal_x("<html><body><p><a href='http://blade.nagaokaut.ac.jp/cgi-bin/scat.rb/ruby/ruby-talk/12345'>[ruby-talk:12345]</a></p></body></html>",ret)
    end

    def test_url3
      s = '((<raa:amrita2:template library>))'
      ret = RDAProcessor.new.process(s)
      assert_equal_x("<html><body><p><a href='http://raa.ruby-lang.org/list.rhtml?name=amrita2'>[template library]</a></p></body></html>",ret)
    end

    def test_kbd
      s = '((%testx a bbb ccc%))'
      t = Amrita2::TemplateText.new "<div><amrita:erb id='testx'><%= $_.join(':') %></amrita:erb></div>"
      t.use_erb(binding)
      t.define_parts(:testx)
      ret = RDAProcessor.new(:exttmpl=>t).process(s)
      assert_equal_x("<html><body><p>a:bbb:ccc\n</p></body></html>",ret)
    end


    def test_blockquote
      s = <<-END
--- blockquote "http://www.ruby-lang.org/" "Ruby Home Page"
    aaaa

bbbb
END
      ret = RDAProcessor.new.process(s)
      assert_equal_x("<html><body><blockquote cite='http://www.ruby-lang.org/'><p>aaaa </p><p>(<a href='http://www.ruby-lang.org/'>Ruby Home Page</a>)</p></blockquote><p>bbbb\n</p></body></html>", ret)
    end

    def test_list
      s = <<-END
* 1
* 2
  (1) a
      aaa
  (2) b
  zzz
END
      ret = RDAProcessor.new.process(s)
      assert_equal_x("<html><body><ul><li>1 </li><li><p>2\n</p><ol><li>a       aaa </li><li>b </li></ol><p>zzz\n</p></li></ul></body></html>", ret)
    end

    def test_list2
      s = <<-END
* ((<a|URL:./a>))
* ((<b|URL:./b>))
END
      ret = RDAProcessor.new.process(s)
      assert_equal_x("<html><body><ul><li><a href='./a'>a</a></li><li><a href='./b'>b</a></li></ul></body></html>", ret)
    end

    def test_desclist
      s = <<-END
:a
   aaaa
:b
   bbbb
   bbbb
END
      ret = RDAProcessor.new.process(s)
      assert_equal_x("<html><body><dl><dt>a</dt><dd>aaaa </dd><dt>b</dt><dd>bbbb bbbb </dd></dl></body></html>", ret)
    end

    def test_header
      s = '= title ttttt'
      ret = RDAProcessor.new.process(s)
      assert_equal_x("<html><body><h1>title ttttt</h1></body></html>",ret)

      s = '== title ttttt'
      ret = RDAProcessor.new.process(s)
      assert_equal_x("<html><body><h2>title ttttt</h2></body></html>",ret)

      s = '=== title ttttt'
      ret = RDAProcessor.new.process(s)
      assert_equal_x("<html><body><h3>title ttttt</h3></body></html>",ret)
    end


    def test_document
      s = <<-END
= header1
== header12
= header2
 
* ((<hello sample|URL:./hello>))
* ((<hello sample source|URL:./hello.rb>))

== header22

aaa
bbb

* ccc((*eeee*))

END
      ret = RDAProcessor.new.process(s)
      assert_equal_x("<html><body><h1>header1</h1><h2>header12</h2><h1>header2</h1><ul><li><a href='./hello'>hello sample</a></li><li><a href='./hello.rb'>hello sample source</a></li></ul><h2>header22</h2><p>aaa\nbbb\n</p><ul><li>ccc<em>*eeee*</em></li></ul></body></html>", ret)
    end      

    def test_head
      s = 'a'
      ret = RDAProcessor.new.process(s, :title=>"ttt", :css=>"default.css")
      assert_equal_x("<html><head><title>ttt</title><link href='default.css' rel='stylesheet' type='text/css'></link></head><body><p>a\n</p></body></html>",ret)
    end

    def test_ext
      exttmpl = TemplateText.new <<-END
      <div>
        <a id='rubytalk' href="http://blade.nagaokaut.ac.jp/cgi-bin/scat.rb/ruby/ruby-talk/$1">((ruby-talk:$1))</a>
      </div>
      END

      exttmpl.element_option[:rubytalk] = :use_args
      exttmpl.define_parts(:rubytalk)
      s = '((<rubytalk:12345>))'
      ret = RDAProcessor.new(:exttmpl=>exttmpl).process(s)
      assert_equal_x("<html><body><p><a href='http://blade.nagaokaut.ac.jp/cgi-bin/scat.rb/ruby/ruby-talk/12345'>((ruby-talk:12345))</a></p></body></html>",ret)
    end

    def test_ext_erb
      exttmpl = TemplateText.new <<-END
      <div>
        <amrita:erb id='plus1'>
           <%= $_.to_i + 1 %>
        </amrita:erb>
      </div>
      END

      exttmpl.use_erb(binding)
      exttmpl.define_parts(:plus1)
      s = '((<plus1:1>))'
      ret = RDAProcessor.new(:exttmpl=>exttmpl).process(s)
      assert_equal_x("<html><body><p>\n           2\n        \n</p></body></html>",ret)
    end

    def test_ext_erb2
      exttmpl = TemplateText.new <<-END
      <div>
        <amrita:erb id='upcase'>
           <%= $_.first.upcase %>
        </amrita:erb>
      </div>
      END

      exttmpl.use_erb(binding)
      exttmpl.define_parts(:upcase)
      s = <<-END
--- upcase
    aaaa 
    bbbb
bbbb
END
      ret = RDAProcessor.new(:exttmpl=>exttmpl).process(s)
      assert_equal_x("<html><body>\n    \n           AAAA  BBBB \n        <p>bbbb\n</p></body></html>",
                     ret)

    end

    def test_ext_erb3
      exttmpl = TemplateText.new <<-'END'
      <div>
        <amrita:erb id='code'>
          <%
            src = $_.first.gsub(%r[<p>#\s*</p>], "")
            src.gsub!(%r[\A<pre>], "")
            src.gsub!(%r[</pre>\Z], "")
          %>
          <pre><code>
          <%= src %>
          </code></pre>
        </amrita:erb>
      </div>
      END

      exttmpl.use_erb(binding)
      #exttmpl.debug_source = STDOUT
      exttmpl.define_parts(:code)
      s = <<-END
--- code
    #
       aaaa 
       bbbb
bbbb
END
      ret = RDAProcessor.new(:exttmpl=>exttmpl).process(s)
      assert_equal_x("<html><body><pre><code>\n          aaaa \nbbbb\n          </code></pre><p>bbbb\n</p></body></html>", 
                     ret)
    end

    def test_ext_erb_many_param
      exttmpl = TemplateText.new <<-END
      <div>
        <amrita:erb id='sum'>
           <%= $_.inject(0) { |sum, x| sum += x.to_i} %>
        </amrita:erb>
      </div>
      END

      exttmpl.use_erb(binding)
      exttmpl.element_option[:sum] = :use_args
      exttmpl.define_parts(:sum)

      s = "--- sum 1 2 "
      ret = RDAProcessor.new(:exttmpl=>exttmpl).process(s)
      assert_equal_x("<html><body>\n    \n           3\n        \n    \n    \n  </body></html>",
                     ret)

      s = "--- sum 1 2 3 \n    4"
      ret = RDAProcessor.new(:exttmpl=>exttmpl).process(s)
      assert_equal_x("<html><body>\n    \n           10\n        \n    \n    \n  </body></html>",
                     ret)
    end
  end
end

if __FILE__ == $0
  include Amrita2RDTest
  cname = ARGV.shift
  if cname
    method = ARGV.shift
    if method
      eval "Test::Unit::UI::Console::TestRunner.run(#{cname}.new(\"#{method}\"))"
    else
      cls = eval(cname)
      Test::Unit::UI::Console::TestRunner.run(cls)
    end
  else
    suite = Test::Unit::TestSuite.new
    suite << TestRD.suite
    Test::Unit::UI::Console::TestRunner.run(suite)
  end
end

# ruby -I~/amrita2/lib ~/amrita2/test/test_rd.rb TestRD test_table |ruby -pe '$_.sub!(/^ */, "") ' 
